@@ -8,11 +8,11 @@ __credits__ = ["Jai Ram Rideout",
                "Greg Caporaso",
                "Jeremy Widmann"]
 __license__ = "GPL"
-__version__ = "1.7.0"
+__version__ = "1.8.0"
 __maintainer__ = "Jai Ram Rideout"
 __email__ = "jai.rideout@gmail.com"
-__status__ = "Release"
 
+from collections import defaultdict
 from numpy import array
 from qiime.pycogent_backports.test import is_symmetric_and_hollow
 from qiime.parse import group_by_field
@@ -396,3 +396,217 @@ def _get_groupings(dist_matrix_header, dist_matrix, groups, within=True,
                 if vals:
                     result.append((row_group, col_group, vals))
     return result
+
+
+def extract_per_individual_states_from_sample_metadata(
+      sample_metadata,
+      state_category,
+      state_values,
+      individual_identifier_category,
+      filter_missing_data=True):
+    """
+    sample_metadata : 2d dictionary mapping sample ids to metadata (as 
+     returned from qiime.parse.parse_mapping_file_to_dict)
+    state_category: metadata category name describing state of interest
+     (usually something like 'TreatmentState') as a string
+    state_values: ordered list of values of interest in the state_category
+     metadata entry (usually something like ['PreTreatment','PostTreatment'])
+    individual_identifier_category: metadata category name describing the
+     individual (usually something like 'PersonalID') as a string
+    filter_missing_data: if True, an individual is excluded 
+     from the result object if any of it's values are None. This can occur
+     when there is no sample for one or more of the state values for an
+     individual. This is True by default.
+    
+    returns {'individual-identifier':
+               [sample-id-at-state-value1,
+                sample-id-at-state-value2,
+                sample-id-at-state-value3,
+                ...],
+              ...
+             }
+    """
+    # prep the result object, which will be a dict of lists
+    len_state_values = len(state_values)
+    def inner_dict_constructor():
+        return [None] * len_state_values
+    results = defaultdict(inner_dict_constructor)
+    
+    for sample_id, metadata in sample_metadata.items():
+        try:
+            individual_id = metadata[individual_identifier_category]
+        except KeyError:
+            raise KeyError, \
+             "%s is not a sample metadata category." %\
+             individual_identifier_category
+        try:
+            state_value = metadata[state_category]
+        except KeyError:
+            raise KeyError, \
+             "%s is not a sample metadata category." %\
+             state_category
+             
+        try:
+            state_index = state_values.index(state_value)
+        except ValueError:
+            # hit a state that is in the mapping file but not in 
+            # state_values - this is silently ignored
+            continue
+        
+        results[individual_id][state_index] = sample_id
+        
+    if filter_missing_data:
+        # delete individual results if sample ids corresponding to
+        # any of the states are missing
+        for individual_id, sample_ids in results.items():
+            if None in sample_ids:
+                del results[individual_id]
+    return results
+
+def extract_per_individual_state_metadatum_from_sample_metadata(
+        sample_metadata,
+        state_category,
+        state_values,
+        individual_identifier_category,
+        metadata_category,
+        process_f=float):
+    """
+    sample_metadata : 2d dictionary mapping sample ids to metadata (as 
+     returned from qiime.parse.parse_mapping_file_to_dict)
+    state_category: metadata category name describing state of interest
+     (usually something like 'TreatmentState') as a string
+    state_values: ordered list of values of interest in the state_category
+     metadata entry (usually something like ['PreTreatment','PostTreatment'])
+    individual_identifier_category: metadata category name describing the
+     individual (usually something like 'PersonalID') as a string
+    metadata_category: metadata category to extract from sample_metadata
+    process_f: function to apply to metadata values (default: float)
+    
+    returns {'individual-identifier':
+               [state-1-metadata-value,
+                state-2-metadata-value,
+                ...],
+              ...
+             }
+    """
+    per_individual_states = extract_per_individual_states_from_sample_metadata(
+     sample_metadata,
+     state_category,
+     state_values,
+     individual_identifier_category,
+     filter_missing_data=True)
+    
+    results = {}
+    for individual_id, sample_ids in per_individual_states.items():
+        per_state_metadata_values = []
+        for sample_id in sample_ids:
+            try:
+                sample_metadata_value = sample_metadata[sample_id][metadata_category]
+            except KeyError:
+                raise KeyError, \
+                 "%s is not a sample metadata category." % metadata_category
+            try:
+                v = process_f(sample_metadata_value)
+            except ValueError, e:
+                v = None
+            per_state_metadata_values.append(v)
+        results[individual_id] = per_state_metadata_values
+    return results
+
+def extract_per_individual_state_metadata_from_sample_metadata(
+        sample_metadata,
+        state_category,
+        state_values,
+        individual_identifier_category,
+        metadata_categories,
+        process_f=float):
+    """
+    sample_metadata : 2d dictionary mapping sample ids to metadata (as 
+     returned from qiime.parse.parse_mapping_file_to_dict)
+    state_category: metadata category name describing state of interest
+     (usually something like 'TreatmentState') as a string
+    state_values: ordered list of values of interest in the state_category
+     metadata entry (usually something like ['PreTreatment','PostTreatment'])
+    individual_identifier_category: metadata category name describing the
+     individual (usually something like 'PersonalID') as a string
+    metadata_categories: metadata categories to extract from sample_metadata
+    process_f: function to apply to metadata values (default: float)
+    
+    returns {'metadata-category-1':
+              {'individual-identifier-1':
+               [difference-in-metadata-value-bw-states-2-and-1,
+                difference-in-metadata-value-bw-states-3-and-2,
+                ...],
+               'individual-identifier-2:
+               [difference-in-metadata-value-bw-states-2-and-1,
+                difference-in-metadata-value-bw-states-3-and-2,
+                ...],
+               }
+              ...
+              }
+    """
+    results = {}
+    for metadata_category in metadata_categories:
+        results[metadata_category] = \
+         extract_per_individual_state_metadatum_from_sample_metadata(
+           sample_metadata,
+           state_category,
+           state_values,
+           individual_identifier_category,
+           metadata_category,
+           process_f)
+    return results
+
+def extract_per_individual_state_metadata_from_sample_metadata_and_biom(
+        sample_metadata,
+        biom_table,
+        state_category,
+        state_values,
+        individual_identifier_category,
+        observation_ids=None):
+    """
+    sample_metadata : 2d dictionary mapping sample ids to metadata (as 
+     returned from qiime.parse.parse_mapping_file_to_dict)
+    biom_table: biom table object containing observation counts for 
+     samples in sample_metadata
+    state_category: metadata category name describing state of interest
+     (usually something like 'TreatmentState') as a string
+    state_values: ordered list of values of interest in the state_category
+     metadata entry (usually something like ['PreTreatment','PostTreatment'])
+    individual_identifier_category: metadata category name describing the
+     individual (usually something like 'PersonalID') as a string
+    observation_ids: observations (usually OTUs) to extract from biom_table
+     (default is all)
+    
+    returns {'otu1':
+              {'individual-identifier-1:
+               [difference-in-otu1-abundance-bw-states-2-and-1,
+                difference-in-otu1-abundance-bw-states-3-and-2,
+                ...],
+               'individual-identifier-2:
+               [difference-in-otu1-abundance-bw-states-2-and-1,
+                difference-in-otu1-abundance-bw-states-3-and-2,
+                ...],
+               }
+              ...
+              }
+    """
+    per_individual_states = extract_per_individual_states_from_sample_metadata(
+     sample_metadata,
+     state_category,
+     state_values,
+     individual_identifier_category,
+     filter_missing_data=True)
+    results = {}
+    if observation_ids is None:
+        observation_ids = biom_table.ObservationIds
+    for observation_id in observation_ids:
+        observation_data = biom_table.observationData(observation_id)
+        results[observation_id] = {}
+        for individual_id, sample_ids in per_individual_states.items():
+            per_state_metadata_values = []
+            for sample_id in sample_ids:
+                sample_index = biom_table.getSampleIndex(sample_id)
+                per_state_metadata_values.append(observation_data[sample_index])
+            results[observation_id][individual_id] = per_state_metadata_values
+    return results
