@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 __author__ = "Greg Caporaso"
 __copyright__ = "Copyright 2011, The QIIME Project" 
-__credits__ = ["Rob Knight","Greg Caporaso", "Kyle Bittinger","Jens Reeder", "William Walters", "Jose Carlos Clemente Litran", "Adam Robbins-Pianka"]
+__credits__ = ["Rob Knight","Greg Caporaso", "Kyle Bittinger","Jens Reeder",
+                "William Walters", "Jose Carlos Clemente Litran",
+                "Adam Robbins-Pianka", "Jose Antonio Navas Molina"]
 __license__ = "GPL"
-__version__ = "1.7.0"
+__version__ = "1.8.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
-__status__ = "Release"
 
 """Contains code for OTU picking, using several techniques.
 
@@ -31,11 +32,11 @@ from cogent.util.misc import remove_files
 from cogent import LoadSeqs, DNA, Alignment
 from cogent.util.trie import build_prefix_map
 from cogent.util.misc import flatten
-from cogent.app.uclust import get_clusters_from_fasta_filepath
 
 from qiime.util import FunctionWithParams, get_tmp_filename, get_qiime_temp_dir
 from qiime.sort import sort_fasta_by_abundance
 from qiime.parse import fields_to_dict
+from qiime.pycogent_backports.uclust import get_clusters_from_fasta_filepath
 from qiime.pycogent_backports.usearch import (usearch_qf,
  usearch61_denovo_cluster, usearch61_ref_cluster)
 
@@ -1238,10 +1239,10 @@ class Usearch610DeNovoOtuPicker(UclustOtuPickerBase):
          'usearch61_sort_method':'abundance',
          'usearch61_maxrejects':32,
          'usearch61_maxaccepts':1,
-         'sizeorder':False
+         'sizeorder':False,
+         'threads':1.0
          }
          
-         
         _params.update(params)
         OtuPicker.__init__(self, _params)
     
@@ -1265,6 +1266,7 @@ class Usearch610DeNovoOtuPicker(UclustOtuPickerBase):
          otherwise a dict is returned with data.
 
         """
+        
         # perform de novo clustering
         clusters = usearch61_denovo_cluster(
          seq_path,
@@ -1282,6 +1284,7 @@ class Usearch610DeNovoOtuPicker(UclustOtuPickerBase):
          usearch61_maxrejects = self.Params['usearch61_maxrejects'],
          usearch61_maxaccepts = self.Params['usearch61_maxaccepts'],
          sizeorder = self.Params['sizeorder'],
+         threads = self.Params['threads'],
          HALT_EXEC=HALT_EXEC
          )
         
@@ -1340,7 +1343,8 @@ class Usearch61ReferenceOtuPicker(UclustOtuPickerBase):
          'usearch61_maxrejects':32,
          'usearch61_maxaccepts':1,
          'sizeorder':False,
-         'suppress_new_clusters':False
+         'suppress_new_clusters':False,
+         'threads':1.0
          }
          
          
@@ -1390,6 +1394,7 @@ class Usearch61ReferenceOtuPicker(UclustOtuPickerBase):
          usearch61_maxaccepts = self.Params['usearch61_maxaccepts'],
          sizeorder = self.Params['sizeorder'],
          suppress_new_clusters = self.Params['suppress_new_clusters'],
+         threads = self.Params['threads'],
          HALT_EXEC=HALT_EXEC
          )
         
@@ -1601,7 +1606,7 @@ class MothurOtuPicker(OtuPicker):
         dump the result to the desired path instead of returning it.
         log_path: path to log, which should include dump of params.
         """
-        app = Mothur(InputHandler='_input_as_path')
+        app = Mothur(InputHandler='_input_as_path', TmpDir=get_qiime_temp_dir())
         app.Parameters['method'].on(self.Params['Algorithm'])
         results = app(seq_path)
         parsed_otus = mothur_parse(results['otu list'])
