@@ -5,12 +5,12 @@ __copyright__ = "Copyright 2011, The QIIME Project"
 __credits__ = ["Rob Knight", "Daniel McDonald", "Greg Caporaso", 
                "Justin Kuczynski", "Jens Reeder", "Catherine Lozupone",
                "Jai Ram Rideout", "Logan Knecht", "Michael Dwan",
-               "Levi McCracken", "Damien Coy", "Yoshiki Vazquez Baeza"] #remember to add yourself if you make changes
+               "Levi McCracken", "Damien Coy", "Yoshiki Vazquez Baeza",
+               "Will Van Treuren"] #remember to add yourself if you make changes
 __license__ = "GPL"
-__version__ = "1.7.0"
+__version__ = "1.8.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
-__status__ = "Release"
 
 
 """Contains general utility code in support of the Qiime project.
@@ -65,11 +65,12 @@ from cogent.app.formatdb import build_blast_db_from_fasta_path,\
 from cogent import LoadSeqs
 from cogent.util.misc import (create_dir, 
                               handle_error_codes)
-from qiime.pycogent_backports.option_parsing import (
-                                        parse_command_line_parameters,
-                                        make_option)
+                              
+from qcli import (parse_command_line_parameters,
+                  make_option,
+                  qcli_system_call)
+                                 
 from qiime.pycogent_backports.test import is_symmetric_and_hollow
-
 from qiime import __version__ as qiime_library_version
 from qiime.parse import (parse_distmat,
                          parse_mapping_file_to_dict,
@@ -1083,22 +1084,8 @@ def get_java_version():
     else:
         return version_line.split()[-1].strip('"')
 
-def qiime_system_call(cmd, shell=True):
-    """Call cmd and return (stdout, stderr, return_value).
-
-    cmd can be either a string containing the command to be run, or a sequence
-    of strings that are the tokens of the command.
-
-    Please see Python's subprocess.Popen for a description of the shell
-    parameter and how cmd is interpreted differently based on its value.
-    """
-    proc = Popen(cmd, shell=shell, universal_newlines=True, stdout=PIPE,
-                 stderr=PIPE)
-    # communicate pulls all stdout/stderr from the PIPEs to 
-    # avoid blocking -- don't remove this line!
-    stdout, stderr = proc.communicate()
-    return_value = proc.returncode
-    return stdout, stderr, return_value
+# retain qiime_system_call function name for backward compatibility
+qiime_system_call = qcli_system_call
 
 def get_qiime_library_version():
     """get QIIME version and the git SHA + current branch (if applicable)"""
@@ -1900,6 +1887,24 @@ class MetadataMap():
         return sorted(self.getSampleMetadata(self.SampleIds[0]).keys()) \
             if len(self.SampleIds) > 0 else []
 
+    def filterSamples(self, sample_ids_to_keep, strict=True):
+        """Remove samples that are not in ``sample_ids_to_keep``.
+
+        If ``strict=True``, a ``ValueError`` will be raised if any of the
+        sample IDs in ``sample_ids_to_keep`` cannot be found in the metadata
+        map.
+        """
+        for sid in self.SampleIds:
+            if sid not in sample_ids_to_keep:
+                del self._metadata[sid]
+
+        if strict:
+            extra_samples = set(sample_ids_to_keep) - set(self.SampleIds)
+
+            if extra_samples:
+                raise ValueError("Could not find the following sample IDs in "
+                                 "metadata map: %s" % ', '.join(extra_samples))
+
 
 class RExecutor(CommandLineApplication):
     """RExecutor application controller
@@ -2096,3 +2101,71 @@ def add_filename_suffix(filepath, suffix):
     root, extension = splitext(basename(filepath))
     return root + suffix + extension
 
+def sync_biom_and_mf(pmf, bt):
+    """Reduce mapping file dict and biom table to shared samples.
+
+    Inputs: 
+     pmf - parsed mapping file from parse_mapping_file_to_dict (nested dict).
+     bt - parse biom table from parse_biom_table (biom table object).
+    Outputs are a bt and pmf that contain only shared samples and a set of 
+    samples that are not shared. If no samples are unshared this final output
+    will be an empty set. 
+    """
+    mf_samples = set(pmf)
+    bt_samples = set(bt.SampleIds)
+    if mf_samples == bt_samples:
+        # agreement, can continue without fear of breaking code
+        return pmf, bt, set()
+    else: 
+        shared_samples = mf_samples.intersection(bt_samples)
+        # check that we shared something
+        assert len(shared_samples)!=0, \
+            "sync_biom_and_mf: No shared samples, no point in continuing."
+        nonshared_samples = mf_samples.union(bt_samples)-shared_samples
+        # remove samples that were in the mapping file but not biom file
+        npmf = {k:v for k,v in pmf.items() if k in shared_samples}
+        # remove samples in the biom table that were not in the mapping file
+        def _f(sv, sid, smd):
+            return sid in shared_samples
+        nbt = bt.filterSamples(_f)
+    return npmf, nbt, nonshared_samples
+
+def biom_taxonomy_formatter(bt, md_key):
+    """Return md strings from bt using md_key in order of bt.ObservationMetadata
+    
+    There are multiple legacy formats for metadata encoding in biom formats 
+    including as lists, dicts, and strings. This function attempts to figure out
+    what form the metadata is in and convert it into a single string. This 
+    function assumes that the metadata is encoded as a single format. It will 
+    break if some of the metadata is e.g., encoded as a dict and some as a list.
+
+    Inputs:
+     bt - biom table object
+     md_key - string, the key to return the metadata from the biom table. 
+    Outputs a list of strings (in order of bt.ObservationMetadata entries) of 
+    metadata. If no metadata could be found using the given key the function 
+    will print a warning and return None. 
+    """
+    if bt.ObservationMetadata is None:
+        print 'No metadata in biom table.'
+        return None
+    else:
+        dtype = bt.ObservationMetadata[0][md_key]
+    if isinstance(dtype, dict):
+        data = []
+        for md in bt.ObservationMetadata:
+            tmp = []
+            for k,v in md[md_key].iteritems():
+                tmp.append('%s_%s' % (k,v))
+            data.append(' '.join(tmp))
+        # data = [' '.join(['%s_%s' % (k,v) for k,v in md[md_key].items()]) for \
+        #     md in bt.ObservationMetadata]
+        return map(str, data)
+    elif isinstance(dtype, list):
+        return map(str, [';'.join(md[md_key]) for md in bt.ObservationMetadata])
+    elif isinstance(dtype, (str, unicode)):
+        return map(str, [md[md_key] for md in bt.ObservationMetadata])
+    else:
+        print ('Metadata format could not be determined or metadata key (%s) '+\
+            'was incorrect. Metadata will not be returned.') % md_key
+        return None
