@@ -4,14 +4,13 @@ from __future__ import division
 
 __author__ = "Greg Caporaso"
 __copyright__ = "Copyright 2011, The QIIME project"
-__credits__ = ["Greg Caporaso"]
+__credits__ = ["Greg Caporaso", "Jai Ram Rideout"]
 __license__ = "GPL"
-__version__ = "1.7.0"
+__version__ = "1.8.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
-__status__ = "Release"
 
-from os.path import split, splitext, getsize, exists
+from os.path import split, splitext, getsize, exists, abspath
 from shutil import copy, rmtree
 from numpy import inf
 from copy import deepcopy
@@ -172,12 +171,14 @@ def assign_tax(repset_fasta_fp,
     try:
         assignment_method = params['assign_taxonomy']['assignment_method']
     except KeyError:
-        assignment_method = 'rdp'
+        assignment_method = 'uclust'
     assign_taxonomy_dir = '%s/%s_assigned_taxonomy' %\
      (output_dir,assignment_method)
     taxonomy_fp = '%s/%s_tax_assignments.txt' % \
      (assign_taxonomy_dir,input_basename)
-    if parallel and (assignment_method == 'rdp' or assignment_method == 'blast'):
+    if parallel and (assignment_method == 'rdp' or
+                     assignment_method == 'blast' or
+                     assignment_method == 'uclust'):
         # Grab the parallel-specific parameters
         try:
             params_str = get_params_str(params['parallel'])
@@ -198,7 +199,7 @@ def assign_tax(repset_fasta_fp,
         # Build the parallel taxonomy assignment command
         assign_taxonomy_cmd = \
          'parallel_assign_taxonomy_%s.py -i %s -o %s -T %s' %\
-         (assignment_method, repset_fasta_fp,assign_taxonomy_dir, params_str)
+         (assignment_method, repset_fasta_fp, assign_taxonomy_dir, params_str)
     else:
         try:
             params_str = get_params_str(params['assign_taxonomy'])
@@ -475,7 +476,7 @@ def iterative_pick_subsampled_open_reference_otus(
                        status_update_callback=status_update_callback)
         
             # Add taxa to otu table
-            add_metadata_cmd = 'add_metadata.py -i %s --observation_mapping_fp %s -o %s --sc_separated taxonomy --observation_header OTUID,taxonomy' %\
+            add_metadata_cmd = 'biom add-metadata -i %s --observation-metadata-fp %s -o %s --sc-separated taxonomy --observation-header OTUID,taxonomy' %\
              (tax_input_otu_table_fp,taxonomy_fp,otu_table_w_tax_fp)
             commands.append([("Add taxa to OTU table",add_metadata_cmd)])
         
@@ -615,10 +616,25 @@ def pick_subsampled_open_reference_otus(input_fp,
             filter_fasta_cmd = 'filter_fasta.py -f %s -o %s -s %s -n' %\
              (input_fp,prefiltered_input_fp,prefilter_failures_list_fp)
             commands.append([('Filter prefilter failures from input', filter_fasta_cmd)])
-            
+
+            # Call the command handler on the list of commands
+            command_handler(commands,
+                            status_update_callback,
+                            logger=logger,
+                            close_logger_on_success=False)
+            commands = []
+
             input_fp = prefiltered_input_fp
             input_dir, input_filename = split(input_fp)
             input_basename, input_ext = splitext(input_filename)
+            if getsize(prefiltered_input_fp) == 0:
+                raise ValueError(
+                 "All sequences were discarded by the prefilter. "
+                 "Are the input sequences in the same orientation "
+                 "in your input file and reference file (you can "
+                 "add 'pick_otus:enable_rev_strand_match True' to "
+                 "your parameters file if not)? Are you using the "
+                 "correct reference file?")
             
         ## Build the OTU picking command
         step1_dir = \
@@ -653,6 +669,13 @@ def pick_subsampled_open_reference_otus(input_fp,
     step1_pick_rep_set_cmd = 'pick_rep_set.py -i %s -o %s -f %s' %\
      (step1_otu_map_fp, step1_repset_fasta_fp, input_fp)
     commands.append([('Pick rep set',step1_pick_rep_set_cmd)])
+
+    # Call the command handler on the list of commands
+    command_handler(commands,
+                    status_update_callback,
+                    logger=logger,
+                    close_logger_on_success=False)
+    commands = []
     
     ## Subsample the failures fasta file to retain (roughly) the
     ## percent_subsample
@@ -661,7 +684,13 @@ def pick_subsampled_open_reference_otus(input_fp,
     subsample_fasta(step1_failures_fasta_fp,
                     step2_input_fasta_fp,
                     percent_subsample)
-    
+
+    logger.write('# Subsample the failures fasta file using API \n' + 
+        'python -c "import qiime; qiime.util.subsample_fasta' + 
+        '(\'%s\', \'%s\', \'%f\')\n\n"' % (abspath(step1_failures_fasta_fp),
+                                          abspath(step2_input_fasta_fp),
+                                          percent_subsample))
+
     ## Prep the OTU picking command for the subsampled failures
     step2_dir = '%s/step2_otus/' % output_dir
     step2_cmd = pick_denovo_otus(step2_input_fasta_fp,
@@ -714,8 +743,10 @@ def pick_subsampled_open_reference_otus(input_fp,
                                      logger)
         step4_otu_map_fp = '%s/failures_failures_otus.txt' % step4_dir
         commands.append([('Pick de novo OTUs on step3 failures', step4_cmd)])
-        # Merge the otu maps
-        cat_otu_tables_cmd = 'cat %s %s %s >> %s' %\
+        # Merge the otu maps, note that we are explicitly using the '>' operator
+        # otherwise passing the --force flag on the script interface would
+        # append the newly created maps to the map that was previously created
+        cat_otu_tables_cmd = 'cat %s %s %s > %s' %\
              (step1_otu_map_fp,step3_otu_map_fp,step4_otu_map_fp,merged_otu_map_fp)
         commands.append([('Merge OTU maps',cat_otu_tables_cmd)])
         step4_repset_fasta_fp = '%s/step4_rep_set.fna' % step4_dir
@@ -724,8 +755,10 @@ def pick_subsampled_open_reference_otus(input_fp,
         commands.append([('Pick representative set for subsampled failures',step4_rep_set_cmd)])
         
     else:
-        # Merge the otu maps
-        cat_otu_tables_cmd = 'cat %s %s >> %s' %\
+        # Merge the otu maps, note that we are explicitly using the '>' operator
+        # otherwise passing the --force flag on the script interface would
+        # append the newly created maps to the map that was previously created
+        cat_otu_tables_cmd = 'cat %s %s > %s' %\
              (step1_otu_map_fp,step3_otu_map_fp,merged_otu_map_fp)
         commands.append([('Merge OTU maps',cat_otu_tables_cmd)])    
         # Move the step 3 failures file to the top-level directory
@@ -742,6 +775,12 @@ def pick_subsampled_open_reference_otus(input_fp,
     # Filter singletons from the otu map
     otu_no_singletons_fp = '%s/final_otu_map_mc%d.txt' % (output_dir,min_otu_size)
     otus_to_keep = filter_otus_from_otu_map(otu_fp,otu_no_singletons_fp,min_otu_size)
+
+    logger.write('# Filter singletons from the otu map using API \n' + 
+          'python -c "import qiime; qiime.filter.filter_otus_from_otu_map' + 
+          '(\'%s\', \'%s\', \'%d\')"\n\n' % (abspath(otu_fp),
+                                            abspath(otu_no_singletons_fp),
+                                            min_otu_size))
     
     ## make the final representative seqs file and a new refseqs file that 
     ## could be used in subsequent otu picking runs.
@@ -761,10 +800,14 @@ def pick_subsampled_open_reference_otus(input_fp,
     for otu_id, seq in MinimalFastaParser(open(step1_repset_fasta_fp,'U')):
             if otu_id.split()[0] in otus_to_keep:
                 final_repset_f.write('>%s\n%s\n' % (otu_id,seq))
+    logger.write('# Write non-singleton otus representative sequences ' + 
+      'from step1 to the final rep set file: %s\n\n' % final_repset_fp)
     # copy the full input refseqs file to the new refseqs_fp
     copy(refseqs_fp,new_refseqs_fp)
     new_refseqs_f = open(new_refseqs_fp,'a')
     new_refseqs_f.write('\n')
+    logger.write('# Copy the full input refseqs file to the new refseq file\n' +
+          'cp %s %s\n\n' % (refseqs_fp, new_refseqs_fp))
     # iterate over all representative sequences from step2 and step4 and write 
     # those corresponding to non-singleton otus to the final representative set
     # file and the new reference sequences file.
@@ -779,6 +822,9 @@ def pick_subsampled_open_reference_otus(input_fp,
                 final_repset_f.write('>%s\n%s\n' % (otu_id,seq))
     new_refseqs_f.close()
     final_repset_f.close()
+    logger.write('# Write non-singleton otus representative sequences from ' +
+      'step 2 and step 4 to the final representative set and the new reference'+
+      ' set (%s and %s respectively)\n\n' % (final_repset_fp, new_refseqs_fp))
     
     # Prep the make_otu_table.py command
     otu_table_fp = '%s/otu_table_mc%d.biom' % (output_dir,min_otu_size)
@@ -829,8 +875,7 @@ def pick_subsampled_open_reference_otus(input_fp,
                        status_update_callback=status_update_callback)
         
             # Add taxa to otu table
-            # Add taxa to otu table
-            add_metadata_cmd = 'add_metadata.py -i %s --observation_mapping_fp %s -o %s --sc_separated taxonomy --observation_header OTUID,taxonomy' %\
+            add_metadata_cmd = 'biom add-metadata -i %s --observation-metadata-fp %s -o %s --sc-separated taxonomy --observation-header OTUID,taxonomy' %\
              (tax_input_otu_table_fp,taxonomy_fp,otu_table_w_tax_fp)
             commands.append([("Add taxa to OTU table",add_metadata_cmd)])
         
