@@ -6,10 +6,9 @@ __author__ = "Greg Caporaso"
 __copyright__ = "Copyright 2011, The QIIME project"
 __credits__ = ["Greg Caporaso", "Kyle Bittinger", "Justin Kuczynski"]
 __license__ = "GPL"
-__version__ = "1.7.0"
+__version__ = "1.8.0"
 __maintainer__ = "Greg Caporaso"
 __email__ = "gregcaporaso@gmail.com"
-__status__ = "Release"
 
 from os.path import split, splitext, abspath
 from qiime.util import (get_qiime_scripts_dir,
@@ -176,12 +175,14 @@ def run_pick_de_novo_otus(input_fp,
     try:
         assignment_method = params['assign_taxonomy']['assignment_method']
     except KeyError:
-        assignment_method = 'rdp'
+        assignment_method = 'uclust'
     assign_taxonomy_dir = '%s/%s_assigned_taxonomy' %\
      (output_dir,assignment_method)
     taxonomy_fp = '%s/%s_rep_set_tax_assignments.txt' % \
      (assign_taxonomy_dir,input_basename)
-    if parallel and (assignment_method == 'rdp' or assignment_method == 'blast'):
+    if parallel and (assignment_method == 'rdp' or
+                     assignment_method == 'blast' or
+                     assignment_method == 'uclust'):
         # Grab the parallel-specific parameters
         try:
             params_str = get_params_str(params['parallel'])
