@@ -9,10 +9,9 @@ __credits__ =  ["Rob Knight", "Micah Hamady", "Greg Caporaso",
                 "Jens Reeder", "Jose Antonio Navas Molina",
                 "Jai Ram Rideout"] #remember to add yourself
 __license__ = "GPL"
-__version__ = "1.7.0"
+__version__ = "1.8.0"
 __maintainer__ = "William Walters"
 __email__ = "william.a.walters@colorado.edu"
-__status__ = "Release"
  
 
 from qiime.util import parse_command_line_parameters, get_options_lookup
@@ -370,10 +369,6 @@ def main():
         except IOError:
             raise IOError,('Unable to open file %s ' % q +' please check '+\
              'filepath and read permissions.')
-        if not q.endswith('qual'):
-            stderr.write(
-            "Qual file does not end with .qual: is it really a qual file?\n%s\n" 
-            % q)
 
     for f in fasta_files:
         try:
@@ -382,10 +377,6 @@ def main():
         except IOError:
             raise IOError,('Unable to open file %s ' % f +' please check '+\
              'filepath and read permissions.')
-        if not (f.endswith('fasta') or f.endswith('fna')):
-            stderr.write(
-            "Fasta file does not end with .fna: is it really a seq file?\n%s\n" 
-            % f)
     
     preprocess(fasta_files, qual_files, mapping_file,
                barcode_type=opts.barcode_type,
