@@ -6,23 +6,26 @@ __copyright__ = "Copyright 2012, The QIIME project"
 __credits__ = ["Jai Ram Rideout", "Michael Dwan", "Logan Knecht",
                "Damien Coy", "Levi McCracken", "Andrew Cochran"]
 __license__ = "GPL"
-__version__ = "1.7.0"
+__version__ = "1.8.0"
 __maintainer__ = "Jai Ram Rideout"
 __email__ = "jai.rideout@gmail.com"
-__status__ = "Release"
 
 """Test suite for classes, methods and functions of the stats module."""
 
+from shutil import rmtree
+from os.path import exists, join
 from string import digits
 from cogent.util.unit_test import TestCase, main
-from numpy import array, asarray, roll, median
+from cogent.util.misc import remove_files, create_dir
+from numpy import array, asarray, roll, median, nan
 from numpy.random import permutation, shuffle
-
+from biom.parse import parse_biom_table
 from qiime.stats import (all_pairs_t_test, _perform_pairwise_tests,
         Anosim, Best, CategoryStats, CorrelationStats, DistanceMatrixStats,
         MantelCorrelogram, Mantel, PartialMantel, Permanova, quantile,
-        _quantile)
-from qiime.util import DistanceMatrix, MetadataMap
+        _quantile,paired_difference_analyses)
+from qiime.util import (DistanceMatrix, MetadataMap, get_qiime_temp_dir,
+                        get_tmp_filename)
 
 
 class TestHelper(TestCase):
@@ -296,25 +299,25 @@ foo	bar	N/A	N/A	N/A	N/A	N/A
         """Test on valid dataset w/ multiple comps."""
         # Verified with R's t.test function.
         exp = [['foo', 'bar', -6.5999999999999996, 0.0070804795641244006,
-            0.021241438692373202, None, None], ['foo', 'baz',
+            0.021241438692373202, nan, nan], ['foo', 'baz',
             -9.7979589711327115, 0.00060818494446333643, 0.0018245548333900093,
-            None, None], ['bar', 'baz', -3.0, 0.05766888562243732,
-            0.17300665686731195, None, None]]
+            nan, nan], ['bar', 'baz', -3.0, 0.05766888562243732,
+            0.17300665686731195, nan, nan]]
         obs = _perform_pairwise_tests(self.labels2, self.dists2, 'two-sided',
                                       0)
         self.assertFloatEqual(obs, exp)
 
     def test_perform_pairwise_tests_too_few_obs(self):
         """Test on dataset w/ too few observations."""
-        exp = [['foo', 'bar', None, None, None, None, None], ['foo', 'baz',
+        exp = [['foo', 'bar', nan, nan, nan, nan, nan], ['foo', 'baz',
             -7.794228634059948, 0.008032650971672552, 0.016065301943345104,
-            None, None], ['bar', 'baz',
+            nan, nan], ['bar', 'baz',
             -2.598076211353316, 0.060844967173160069, 0.12168993434632014,
-            None, None]]
+            nan, nan]]
         obs = _perform_pairwise_tests(self.labels3, self.dists3, 'low', 0)
         self.assertFloatEqual(obs, exp)
 
-        exp = [['foo', 'bar', None, None, None, None, None]]
+        exp = [['foo', 'bar', nan, nan, nan, nan, nan]]
         obs = _perform_pairwise_tests(['foo', 'bar'], [[], [1, 2, 4]], 'high',
                                       20)
         self.assertFloatEqual(obs, exp)
@@ -1072,8 +1075,20 @@ class BestTests(TestHelper):
                        'CMIN_RATE = 9', 'LONGITUDE = 10',
                        'LATITUDE = 11']}
 
+        # the difference caused by the new spearmans_rho function is in the 
+        # final decimal place of the rho_vals. its not a significant difference, 
+        # but the test is failing because we are doing a dict comparison for 
+        # float values. 
         obs = self.best()
-        self.assertTrue(exp == obs)
+        self.assertEqual(exp['method_name'], obs['method_name'])
+        self.assertEqual(exp['num_vars'], obs['num_vars'])
+        self.assertEqual(exp['vars'], obs['vars'])
+        for i,j in zip(exp['rho_vals'], obs['rho_vals']):
+            self.assertFloatEqual(i[0], j[0])
+            self.assertEqual(i[1], j[1])
+        # check that the keys are the same since we have checked all the values
+        # we expect to be there are the same
+        self.assertTrue(set(exp.keys())==set(obs.keys()))
 
 
 class MantelCorrelogramTests(TestHelper):
@@ -1400,10 +1415,6 @@ class MantelCorrelogramTests(TestHelper):
         self.assertEqual(obs_ax.get_title(), "Mantel Correlogram")
         self.assertEqual(obs_ax.get_xlabel(), "Distance class index")
         self.assertEqual(obs_ax.get_ylabel(), "Mantel correlation statistic")
-        self.assertFloatEqual(obs_ax.get_xticks(),
-            [0., 0.2, 0.4, 0.6, 0.8, 1.0])
-        self.assertFloatEqual(obs_ax.get_yticks(),
-            [0., 0.2, 0.4, 0.6, 0.8, 1.0])
 
 
 class MantelTests(TestHelper):
@@ -1638,6 +1649,150 @@ class TopLevelTests(TestCase):
         sample_data.sort()
         self.assertFloatEqual(_quantile(sample_data, 0.10), 0.283062154)
 
+class PairedDifferenceTests(TestCase):
+    
+    def setUp(self):
+        self.personal_ids_to_state_values1 = \
+          {'firmicutes-abundance':
+            {'subject1':[0.45,0.55],
+             'subject2':[0.11,0.52]},
+           'bacteroidetes-abundance':
+             {'subject1':[0.28,0.21],
+              'subject2':[0.11,0.01]}
+           }
+        self.personal_ids_to_state_values2 = \
+          {'firmicutes-abundance':
+            {'subject1':[0.45,0.55],
+             'subject2':[0.11,None]},
+           'bacteroidetes-abundance':
+             {'subject1':[0.28,0.21],
+              'subject2':[0.11,0.01]}
+           }
+        self.files_to_remove = []
+        self.dirs_to_remove = []
+        tmp_dir = get_qiime_temp_dir()
+        self.test_out = get_tmp_filename(tmp_dir=tmp_dir,
+                                         prefix='qiime_paired_diff_tests_',
+                                         suffix='',
+                                         result_constructor=str)
+        self.dirs_to_remove.append(self.test_out)
+        create_dir(self.test_out)
+
+    def tearDown(self):
+        
+        remove_files(self.files_to_remove)
+        # remove directories last, so we don't get errors
+        # trying to remove files which may be in the directories
+        for d in self.dirs_to_remove:
+            if exists(d):
+                rmtree(d)
+    
+    def test_paired_difference_analyses(self):
+        """paired_difference_analyses functions as expected
+        """
+        actual = paired_difference_analyses(
+                                   self.personal_ids_to_state_values1,
+                                   ['firmicutes-abundance',
+                                    'bacteroidetes-abundance'],
+                                   ['Pre','Post'],
+                                   output_dir=self.test_out,
+                                   ymin=0.0,
+                                   ymax=1.0)
+        self.assertTrue(exists(join(self.test_out,
+                        'paired_difference_comparisons.txt')))
+        self.assertTrue(exists(join(self.test_out,'firmicutes-abundance.pdf')))
+        self.assertTrue(exists(join(self.test_out,'bacteroidetes-abundance.pdf')))
+        # three output paths returned
+        self.assertEqual(len(actual[0]),5)
+        # expected t values returned
+        self.assertFloatEqual(actual[1]['firmicutes-abundance'][4],1.645,3)
+        self.assertFloatEqual(actual[1]['bacteroidetes-abundance'][4],-4.500,3)
+
+    def test_paired_difference_analyses_biom_output(self):
+        """paired_difference_analyses generates correct biom tables
+        """
+        actual = paired_difference_analyses(
+                                   self.personal_ids_to_state_values1,
+                                   ['firmicutes-abundance',
+                                    'bacteroidetes-abundance'],
+                                   ['Pre','Post'],
+                                   output_dir=self.test_out,
+                                   ymin=0.0,
+                                   ymax=1.0)
+        biom_table_fp = join(self.test_out,'differences.biom')
+        self.assertTrue(exists(biom_table_fp))
+        self.assertTrue(exists(join(self.test_out,'differences_sids.txt')))
+        table = parse_biom_table(open(biom_table_fp,'U'))
+        self.assertEqualItems(table.SampleIds,['subject1','subject2'])
+        self.assertEqualItems(table.ObservationIds,
+         ['firmicutes-abundance','bacteroidetes-abundance'])
+        self.assertFloatEqual(table
+            [table.getObservationIndex('firmicutes-abundance')]
+            [table.getSampleIndex('subject1')],
+            0.1,2)
+        self.assertFloatEqual(table
+            [table.getObservationIndex('bacteroidetes-abundance')]
+            [table.getSampleIndex('subject1')],
+            -0.1,2)
+        self.assertFloatEqual(table
+            [table.getObservationIndex('firmicutes-abundance')]
+            [table.getSampleIndex('subject2')],
+            0.41,2)
+        self.assertFloatEqual(table
+            [table.getObservationIndex('bacteroidetes-abundance')]
+            [table.getSampleIndex('subject2')],
+            -0.07,2)
+
+        # missing data should raise ValueError
+        self.assertRaises(ValueError, paired_difference_analyses,
+                          self.personal_ids_to_state_values2,
+                          ['firmicutes-abundance',
+                           'bacteroidetes-abundance'],
+                          ['Pre','Post'],
+                          output_dir=self.test_out,
+                          ymin=0.0,
+                          ymax=1.0)
+
+    def test_paired_difference_analyses_wo_ymin_ymax(self):
+        """paired_difference_analyses functions as expected w/o ymin/ymax
+        """
+        # runs successfully with ymin/ymax
+        actual = paired_difference_analyses(
+                                   self.personal_ids_to_state_values1,
+                                   ['firmicutes-abundance',
+                                    'bacteroidetes-abundance'],
+                                   ['Pre','Post'],
+                                   output_dir=self.test_out,
+                                   ymin=None,
+                                   ymax=None)
+        self.assertTrue(exists(join(self.test_out,
+                        'paired_difference_comparisons.txt')))
+        self.assertTrue(exists(join(self.test_out,'firmicutes-abundance.pdf')))
+        self.assertTrue(exists(join(self.test_out,'bacteroidetes-abundance.pdf')))
+        # three output paths returned
+        self.assertEqual(len(actual[0]),5)
+        # expected t values returned
+        self.assertFloatEqual(actual[1]['firmicutes-abundance'][4],1.645,3)
+        self.assertFloatEqual(actual[1]['bacteroidetes-abundance'][4],-4.500,3)
+
+    def test_paired_difference_analyses_analysis_cat_subset(self):
+        """paired_difference_analyses fns w a subset of analysis categories
+        """
+        actual = paired_difference_analyses(
+                                   self.personal_ids_to_state_values1,
+                                   ['firmicutes-abundance'],
+                                   ['Pre','Post'],
+                                   output_dir=self.test_out,
+                                   ymin=0.0,
+                                   ymax=1.0)
+        self.assertTrue(exists(join(self.test_out,
+                        'paired_difference_comparisons.txt')))
+        self.assertTrue(exists(join(self.test_out,'firmicutes-abundance.pdf')))
+        self.assertFalse(exists(join(self.test_out,'bacteroidetes-abundance.pdf')))
+        # three output paths returned
+        self.assertEqual(len(actual[0]),4)
+        # expected t values returned
+        self.assertFloatEqual(actual[1]['firmicutes-abundance'][4],1.645,3)
 
 if __name__ == "__main__":
     main()
